/*
 * PIC16F18877
 * Esempio impostazione config word per usare l'oscillatore interno a 1MHz
 * e successivamente cambiarlo a 8MHz
 * collegare oscilloscopio su RA6 per controllare il clock in uscita
 * se non impostiamo nessun nuovo oscillatore,
 * su RA6 sar presente un segnale alla frequenza di 1/4=0.25MHz (250KHz)
 * se invece impostiamo un nuovo oscillatore,
 * ad esempio 8MHz, avremo 8/4 = 2MHz
 * 
 * (c)2018 Giovanni Bernardo
 * http://www.settorezero.com
 *  
 */

// CONFIG1 - Selezione oscillatore allo start-up
#pragma config FEXTOSC = OFF    // Oscillatore esterno NON abilitato
#pragma config RSTOSC = HFINT1 // Oscillatore all'avvio (dopo un reset): Interno a 1MHz (quindi PLL non attivo)
// se voglio 32MHz senza PLL metto HFINT32. Se voglio il PLL metter HFINTPLL e avr comunque 32MHz
// ma con l'utilizzo del PLL.

#pragma config CLKOUTEN = ON    // Sul pin OSC2 (RA6/CLKOUT)  presente FOSC/4
// se voglio usare anche RA6 come I/O allora metto OFF

#pragma config CSWEN = ON       // Clock Switching abilitato ( possibile definire NOSC e NDIV)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor disabilitato

// CONFIG2 - Sistemi di supervisione
#pragma config MCLRE = ON       // RE3  usato come MCLR (mettere pull-up, non  possibile usare RE3 come input)
#pragma config PWRTE = OFF      // Power-up Timer disabilitato (con l'oscillatore interno non serve)
#pragma config LPBOREN = OFF    // Low-Power BOR Disabilitato
#pragma config BOREN = ON       // Brown-out reset sempre attivo (il bit SBOREN viene ignorato)
#pragma config BORV = LO        // Tensione di Brown-out Reset (VBOR) impostata a 1.9V sulla variante LF, e 2.45V sulla F
#pragma config ZCD = OFF        // Circuito di Zero-cross detect disabilitato all'avvio
#pragma config PPS1WAY = ON     // Peripheral Pin Select possibile una solta volta (il bit PPSLOCK pu essere resettato e settato una sola volta)
#pragma config STVREN = ON      // Reset per Stack Overflow/Underflow abilitato

// CONFIG3 - Windowed Watchdog
#pragma config WDTE = OFF       // WDT non attivo (il bit SWDTEN non viene preso in considerazione)
#pragma config WDTCPS = WDTCPS_31// WDT Period Select bits (Divider ratio 1:65536; software control of WDTPS)
#pragma config WDTCWS = WDTCWS_7// WDT Window Select (finestra sempre aperta (100%); controllo da software, "keyed access" non richiesto (il datasheet non spiega cosa significa!)
#pragma config WDTCCS = SC      // Clock per WDT selezionato da software

// CONFIG4 - Memoria
#pragma config WRT = OFF        // Protezione eeprom dati non attiva (il software pu usare l'eeprom detta anche NVM)
#pragma config SCANE = not_available// modulo scanner non disponibile
#pragma config LVP = OFF        // Programmazione a bassa tensione non attiva

// CONFIG5 - Protezione
#pragma config CP = OFF         // E' possibile leggere la memoria flash dall'esterno
#pragma config CPD = OFF        // E' possibile leggere l'eeprom dati dall'esterno

#include <xc.h>
#define LED LATAbits.LATA0 // ho collegato un led su RA0
// ricordo, come sempre, che questa macro  necessaria unicamente
// alle funzioni di Delay incluse nel compilatore XC8

// qui dico che FOSC  1MHz
#define _XTAL_FREQ  1000000

// qui dico che FOSC  8MHz
// #define _XTAL_FREQ  8000000

//qui dico che FOSC  32MHz
//#define _XTAL_FREQ  32000000

void main(void) 
    {
    // disattivo la funzione analogica su tutti i pin
    ANSELA=0;
    ANSELB=0;
    ANSELC=0;
    ANSELD=0;
    ANSELE=0;
    
    // pin non usati impostati come ingresso
    // pin RA0 e RA6 impostati come uscite (su RA6 c' il clockout come da word di configurazione)
    TRISA=0b10111110;
    TRISB=0xFF;
    TRISC=0xFF;
    TRISD=0xFF;
    TRISE=0xFF;
 
    // settiamo l'oscillatore interno a 8MHz
    // anche se le impostazioni di OSCCON1 sono le stesse di partenza
    // bisogna riscriverle per segnalare che voglio usare un nuovo oscillatore
    //OSCCON1bits.NOSC=0b110; // nuovo oscillatore: interno HF, poi dovr usare OSCFRQ per impostare la frequenza
    //OSCCON1bits.NDIV=0; // non voglio dividere la frequenza (1:1)
    //OSCFRQ=0b011; // Oscillatore interno a 8MHz
    
    while(1)
        {
        LED^=1; // inverto lo stato di LED (ovvero RA0)...
        __delay_ms(200); // ...ogni 200mS (ammesso di aver definito correttamente _XTAL_FREQ)
        }
    }